/**
 * @(#)PolyLine.java - used to store an array of x,y points
 *
 * Created on August 1, 2000, 9:05 PM
 */
 


/** 
 *
 * @author  furyhawk <lookmeow@magix.com.sg>
 * @version 0.02
 */

/**
   ********************************************************************
 CivLog - Copyright (C) 2000 - Wong Teck Meng
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
**********************************************************************
                                                                      */

import java.awt.*;
import java.util.*;

public class PolyLine extends Object {
  
  Vector x;
  Vector y;
  Vector value;
  int numofPoints = 0;
  
  String label = null;
  Color color = Color.black;
  Font font = null;
  
  int maxIndex = 0;   /** should be null */
  int minIndex = 0;   /** should be null */
  long total   = 0;
  float currentX = 0; /** stored current x pos in float for more precision */
  
  private int currentMaxValue = 0;
  private int currentMinValue = Integer.MAX_VALUE;
  
  /** Default PolyLine implicit Constructor */
  public PolyLine () {
    this.x = new Vector();
    this.y = new Vector();
    this.value = new Vector();
    font = new Font(null, Font.PLAIN, 8);
  }
  
  /** Creates new PolyLine */
  public PolyLine (int x1, int y1) {
    this.x = new Vector();
    this.y = new Vector();
    this.value = new Vector();
    addPoint (x1, y1);
    font = new Font (null, Font.PLAIN, 8);
  }
  
  /** Creates new PolyLine */
  public PolyLine (int x1, int y1, int theValue) {
    this.x = new Vector();
    this.y = new Vector();
    this.value = new Vector();
    addPoint (x1, y1, theValue);
    font = new Font (null, Font.PLAIN, 8);
  }
  
  /** need to use this function to calculate Max, Min and Average before using draw*() */
  public void calculateMaxMinAvg () {
    int value;
    for (int i=0; i < numofPoints; i++) {
      value = getValue (i);
      currentMaxValue = Math.max (currentMaxValue, value);
      currentMinValue = Math.min (currentMinValue, value);
      total += value;   /** calculate the total */
      /** a max point */
      if ( currentMaxValue == value )
      {
        maxIndex = i;
      }
      /** a min point */
      if ( currentMinValue == value )
      {
        minIndex = i;
      }
    }
  }
  
  void addPoint (int x1, int y1){
    this.x.addElement ( new Integer (x1) );
    this.y.addElement ( new Integer (y1) );
    this.value.addElement ( new Integer (0) );
    numofPoints++;
  }
  
  void addPoint (int x1, int y1, int theValue){
    this.x.addElement ( new Integer (x1) );
    this.y.addElement ( new Integer (y1) );
    this.value.addElement ( new Integer (theValue) );
    numofPoints++;
  }
  
  /** i supposed it is returning a copy? */
  int getX (int index) {
    Integer x1 = (Integer)this.x.elementAt (index);
    if (x1 == null)
    throw new RuntimeException ("PolyLine.getX(): elementAt("+index+") empty");

    return ( x1.intValue () );
  }
  
  /** i supposed it is returning a copy? */
  int getY (int index) {
    Integer y1 = (Integer)this.y.elementAt (index);
    if (y1 == null)
    throw new RuntimeException ("PolyLine.getY(): elementAt("+index+") empty");
        
    return ( y1.intValue () );
  }
  
    /** i supposed it is returning a copy? */
  int getValue (int index) {
    Integer theValue = (Integer)this.value.elementAt (index);
    if (theValue == null)
    throw new RuntimeException ("PolyLine.getValue(): elementAt("+index+") empty");
        
    return ( theValue.intValue () );
  }
  
  void setLabel (String newLabel) {
    label = new String(newLabel);
  }
  
  void setColor (Color c) {
    color = c;
  }
  
  void drawPolyLine (Graphics g) {
    g.setColor (color);
    for (int i=1; i < numofPoints; i++) {
      g.drawLine (getX (i-1), getY (i-1), getX (i), getY (i));
    }
  }
  
  /** need to calculateMaxMinAvg () before using this function */
  void drawAverage (Graphics g, Dimension d, float scale) {
    /** the draw average line is move to avoid overwriting others */
    g.setColor (Color.cyan);
    /** calculate the average */
    total = total / numofPoints;
    int scaledAverage = (int)(total * scale); /** scale the average for display */
    g.drawLine (0 , d.height - scaledAverage, d.width, d.height - scaledAverage);
    g.setColor (Color.black); /** reset back the color */
    g.drawString ("Avg: " + total, 35, d.height - scaledAverage - 3);
  }
  
  void drawLabel (Graphics g, int windowHeight) {
    int y1;
    g.setColor (Color.black);
    /** if the end point is below the display area, adjust it */
    if ( getY (numofPoints-1) >= windowHeight-15) { /** y1, x, 2*y1 */
      y1 = getY (numofPoints-1) - 30;
    }
    else {
      y1 = getY (numofPoints-1);
    }
    g.drawString (  label, getX (numofPoints-1)-6, y1+15);
  }
  
  /** need to calculateMaxMinAvg () before using this function */
  void drawMaxPoint (Graphics g) {
    /** use a red dot to mark the max point */
    g.setColor (Color.red);
    g.fillOval (getX (maxIndex)-2, getY (maxIndex)-2, 4, 4);
    g.setColor (Color.black);
    g.drawString ( "Max " + currentMaxValue, getX (maxIndex)-45, getY (maxIndex)-3);
  }
    
  /** need to calculateMaxMinAvg () before using this function */
  void drawMinPoint (Graphics g) {
    /** use a red dot to mark the max point */
    g.setColor (Color.red);
    g.fillOval (getX (minIndex)-2, getY (minIndex)-2, 4, 4);
    g.setColor (Color.black);
    /** decide whether to display Min towards the left or right */
    if (getX (minIndex) < 40) {
      g.drawString ( "Min " + currentMinValue, getX (minIndex)+2, getY (minIndex)-15);
    }
    else {
      g.drawString ( "Min " + currentMinValue, getX (minIndex)-40, getY (minIndex)-15);
    }
    Debug.trace ("points: " + numofPoints);
  }

}/** end - PolyLine.java */